part of charted.charts;

class GtTooltip implements ChartTooltip {
  static const TOOLTIP_HORIZONTAL_OFFSET = 20;
  Element host;
  SelectionScope _scope;
  Selection _host;
  ChartArea _chartArea;
  String _config;

  GtTooltip(Element this.host, [this._config = '']) {
    assert(host != null);
  }

  init(ChartArea area) {
    _chartArea = area;
  }

  show(int activeRow, ChartSeries activeSeries, math.Point coordinate){

    if (_scope == null) {
      _scope = new SelectionScope.element(host);
      _host = _scope.selectElements([host]);
    }

    // Clear
    host.children.clear();

    // Create the main tooltip dive and title.
    _host.append('div')
      ..classed('gt-tooltip-title')
      ..text(_chartArea.data.rows.elementAt(activeRow).elementAt(
          _chartArea.config.dimensions.elementAt(0)));



    var activeRowData = _chartArea.data.rows.elementAt(activeRow);
    var measures = activeSeries.measures;

    if (_config == 'showTotal') {
      var formatFunc = _chartArea.data.columns.elementAt(measures.elementAt(0)).
          formatter;
      var total = 0;
      for (var i = 0; i < measures.length; i++) {
        total += activeRowData.elementAt(measures.elementAt(i));
      }
      _host.append('div')
          ..classed('gt-tooltip-total')
          ..text((formatFunc != null) ? formatFunc(total) : total.toString());
    }

    // Create the tooltip items base on the number of measures in the series.
    var items = _host.selectAll('.gt-tooltip-item').dataWithCallback((d, i, c) =>
        activeSeries.measures);
    items.enter.append('div')
          ..classed('gt-tooltip-item')
          ..classedWithCallback('active', (d, i, c) => (i == activeRow));

    // Display the label for the currently active series.
    var tooltipItems = _host.selectAll('.gt-tooltip-item');
    tooltipItems.append('div')
        ..classed('gt-tooltip-item-label')
        ..textWithCallback((d, i, c) => _chartArea.data.columns.elementAt(
            activeSeries.measures.elementAt(i)).label);

    // Display the value of the currently active series
    tooltipItems.append('div')
        ..classed('gt-tooltip-item-value')
        ..textWithCallback((d, i, c) {
      var fmt = _chartArea.data.columns.elementAt(d).formatter;
      var value = _chartArea.data.rows.elementAt(activeRow).elementAt(d);
      return (fmt != null) ? fmt(value) : value.toString();
    });

    _positionAndShowTooltip(coordinate);
  }

  void _positionAndShowTooltip(math.Point coordinate) {
    // Position the tooltip, defaulting to 20px right and
    // half the height of the tooltip above the hovered point.
    var width = host.getBoundingClientRect().width;
    var height = host.getBoundingClientRect().height;
    var windowWidth = window.innerWidth;
    var windowHeight = window.innerHeight;
    var top = 0;
    var left = 0;

    // Display tooltip to the top of the window if tooltip height is larger than
    // window height
    if (height >= windowHeight) {
      top = 0;
    } else {
      if (coordinate.y - height / 2 < 0) {
        // Display tooltip to top of the screen if default position is higher
        // than top of the window.
        top = 0;
      } else if (coordinate.y + height / 2 > windowHeight) {
        // Display tooltip to bottom of the screen if default position is lower
        // than bottom of the window.
        top = windowHeight - height;
      } else {
        // Display tooltip in default height.
        top = coordinate.y - height / 2;
      }
    }

    // Display tooltip to the left of the window if tooltip width is larger than
    // window width.
    if (width >= windowWidth) {
      left = 0;
    } else {
      left = (coordinate.x + TOOLTIP_HORIZONTAL_OFFSET + width) < windowWidth ?
          (coordinate.x + TOOLTIP_HORIZONTAL_OFFSET) : windowWidth - width;
    }
    _host
        ..style('left', '${left}px')
        ..style('top', '${top}px')
        ..style('opacity', '1');
  }

  hide() {
    if (_host == null) {
      return;
    }
    _host.style('opacity', '0');
  }
}
